#include <String.h>
#include <Directory.h>
#include <Entry.h>
#include <Roster.h>
#include "AutoCompleteTextControl.h"

#if defined(__INTEL__)
#include <BeDC.h>
#endif

TAutoCompleteTextControl :: TAutoCompleteTextControl()
:
	BTextControl(BRect(20,20,180,50),"filenamebox","filenamebox","",NULL)
{
	completion_enabled = true;
}


TAutoCompleteTextControl :: TAutoCompleteTextControl(BRect frame, const char* name,
	const char* label, const char* text, BMessage *message)
:
	BTextControl(frame,name,label,text,message)
{
	completion_enabled = true;
}


bool TAutoCompleteTextControl :: CursorAtEndOfRow()
{
	int32 sel_begin=0, sel_end=0;
	TextView()->GetSelection(&sel_begin,&sel_end);
	BString helper = Text();
	if (sel_begin==sel_end && sel_begin==helper.Length())
		return true;
	else
		return false;
}


bool TAutoCompleteTextControl :: NoSelection()
{
	int32 sel_begin=0,sel_end=0;
	TextView()->GetSelection(&sel_begin,&sel_end);
	if (sel_begin==sel_end)
		return true;
	else
		return false;
}


// public - perhaps to answer Ctrl+Space
void TAutoCompleteTextControl :: Complete()
{
	// should'nt this check occur before (in TextChanged()?)
	if (NoSelection())
	{
		// shouldnt i just cahe the hit?
		BString hit = lookup_completion();
		display_hit(hit);
	}
}


void TAutoCompleteTextControl :: TextChanged()
{
	parse_path();
	BString text = Text();
/*	int index = text.FindFirst("~");
	if (index==0)
	{
		text.ReplaceFirst("~","/boot/home");
		SetText(text.String());
	}*/
	if (text.Length()>0 && text[text.Length()-1] != '/' &&
			completion_enabled && CursorAtEndOfRow())
		Complete();
}


void TAutoCompleteTextControl::EnableCompletion()
{
	completion_enabled = true;
}


void TAutoCompleteTextControl::DisableCompletion()
{
	completion_enabled = false;
}


BString TAutoCompleteTextControl::Path()
{
	return currect_path;
}


// attempts to cancel the modification message to prevent
// SetText from sending a modification message.
void TAutoCompleteTextControl :: UpdateText(BString text)
{
	BMessage msg = *(ModificationMessage());
	// note that this function FREEs the previous message, if any.
	SetModificationMessage(NULL);
		TextView()->SetText(text.String());
	BMessage* msg_ptr = new BMessage(msg);
	SetModificationMessage(msg_ptr);
	// no need to free anything (the control will free the BMessage by itself...)
}


//````````````````````````````	$$
// Privates :										$$
//_____________________	$$  


void TAutoCompleteTextControl :: parse_path()
{
	BString path(Text());
	if (path.Length()>0)
	{
		if (path[path.Length()-1] != '/')
		{
			int index = path.FindLast('/');
			if (index>0)
				path.Remove(index+1,path.Length()-index-1);
			else if (index==0)
				path = "/";
		}
		if (path != currect_path)
		{
			currect_path = path;
			get_filelist();
		}
	}
}


void TAutoCompleteTextControl :: get_filelist()
{
	filelist="";
	entry_ref ref;
	if (get_ref_for_path(currect_path.String(),&ref) == B_OK)
	{
		BEntry entry;
		entry.SetTo(&ref,true);
		if (entry.IsDirectory())
		{
			BDirectory dir;
			dir.SetTo(&entry);
			// problem with symlink?
			while(dir.GetNextEntry(&entry,true) == B_OK)
			{
				char name[B_FILE_NAME_LENGTH];
				entry.GetName(name);
				filelist << name;
				if (entry.IsDirectory())
					filelist << "/";
				filelist << "\n";
			}
		}
		// used in lookup_autocompletion for subword search
		filelist.Prepend("\n");
	}
	else filelist="no such path.";
}


// removes everything but the 'leaf' text from the Text().
// ( /boot/home/jonga -> jonga)
BString TAutoCompleteTextControl :: get_subword()
{
	BString path = Text();
	if (path.Length()>0 && path[path.Length()-1] != '/')
	{
		int index = path.FindLast('/');
		// note the >= (to include the first token in the path i.e />>lateralus<< 
 		if (index>=0) 
			path.Remove(0,index+1);
		return path;
	}
	else return BString("");
}


BString TAutoCompleteTextControl :: lookup_completion()
{
	BString result="";
	BString subword = get_subword(); 
	// i add a \n to the begining of the subword to make sure it appears as the begining
	// of an entry_name inside the filelist
	subword.Prepend("\n");
	int index = filelist.FindFirst(subword);
	char c=0;
	if (index >=0)
	{
		// skipping the first \n
		index++;
		c = filelist.ByteAt(index);
		while (c!='\n' && index<=filelist.Length())
		{										/*=??*/		
			result << c;
			index++;
			c = filelist.ByteAt(index);			
		}
	}
	return result;
}


void TAutoCompleteTextControl :: display_hit(BString hit)
{
	if (hit.Length()>0)
	{
		BString new_text = "";
		new_text = Text();
		// note that these is no need for -1 here.
		int sel_begin = new_text.Length(); 
		new_text = "";
		new_text << currect_path << hit;
		// replaces SetText (although seems to have the same effect)
			UpdateText(new_text);
		TextView()->Select(sel_begin,new_text.Length());
		TextView()->ScrollToOffset(new_text.Length());
	}
}

