#include <Application.h>
#include <Button.h>
#include <TextControl.h>
#include <String.h>
#include <Roster.h>
#include <Entry.h>
#include <Alert.h>
#include <Directory.h>
#include <MenuItem.h>
#include <SymLink.h>
#include <FilePanel.h>
#include <Path.h>
#include <stdlib.h>

#if defined(__INTEL__)
#include <BeDC.h>
#endif

#include "snapApp.h"
#include "myFilter.h"
#include "snapWin.h"
#include "AutoCompleteTextControl.h"
#include "layout.cpp"


// service functions Decleration
	void createHistoryLink(const char*,BMenu*);
	void fillHistoryMenu(BMenu*);
	bool existsInMenu(BString , BMenu* );

///`````````````````````````````````````	:
// Public functoins												:
//,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,	:

snapWin :: snapWin()
:
	BWindow(BRect(50,50,350,180),"Snap",
		B_TITLED_WINDOW_LOOK, B_FLOATING_ALL_WINDOW_FEEL,
		B_NOT_RESIZABLE | B_NOT_ZOOMABLE)
{
	// non-control related.
		IsOnTop = true;
		// used to flag the app when Enter was already pressed and 1 execution is successful
		// (prevents other attemps to execute - which cause things to be executed *atleast* twice.
		ignore_enter = false;
	
	//  Settings up all control Rects 
	// Browse
		BRect browseRect(0,0,60,25);
		WVCenter(browseRect);
		browseRect.OffsetBy(0,-15);
		GlueRight(browseRect);
	// editbox
		BRect editboxRect(0,0,160,20);
		editboxRect.right = browseRect.left - 10;
		GlueLeft(editboxRect);
		VCenter(browseRect,editboxRect);
	// Ok, Cancel
		BRect btnRect(0,0,60,25);
		GlueRight(btnRect);
		btnRect.OffsetBy(0,editboxRect.bottom + 5);
	// Status Bar
		BRect statusbarRect = editboxRect;
		statusbarRect.OffsetBy(0,75);
		statusbarRect.bottom += 20;
	// History menuField
		BRect historyRect(0,0,20,20);
		GlueLeft(historyRect);
		VCenter(btnRect, historyRect);
	// Main Menu Field
		BRect mainMenuRect(0,0,60,20);
		GlueRight(mainMenuRect);
		VCenter(editboxRect,mainMenuRect);
		mainMenuRect.OffsetBy(0,-30);

	// instantiations
		view = new BView(Bounds(),"greyView", B_FOLLOW_ALL_SIDES, B_WILL_DRAW);

		mainPopMenu = new BPopUpMenu("MainPopMenu");
	
		mainMenuField = new BMenuField(mainMenuRect,
			"mainMenuField","options",mainPopMenu,true);
	
		browseBtn = new BButton(browseRect,
			"Browse...","Browse...",	new BMessage(BROWSE));

		cancelBtn = new BButton(btnRect,
			"Cancel","Cancel",	new BMessage(CANCEL));
		
		btnRect.OffsetBy(-65,0);
		okBtn = new BButton(btnRect,
			"Ok","Ok",	new BMessage(OK));

		editbox = new TAutoCompleteTextControl(editboxRect,
			"filenamebox","path","",NULL);
			/*new BMessage(ENTER_KEY) No! :) note i have no regular message defined for this one...*/  
			
		statusbar = new BTextView(statusbarRect,
			"statusbar",statusbarRect,B_FOLLOW_ALL, B_WILL_DRAW);

		historyPopMenu = new BPopUpMenu("HistoryPopMenu");

		historyField = new BMenuField(historyRect, "HistoryMenuField",
			"",historyPopMenu,true);

		filePanel = new BFilePanel(B_OPEN_PANEL,NULL,NULL,0,false);
		filePanel->SetTarget(this);

	// settings up all sorts of view colors
		rgb_color grey = {212,212,212,212};
		view->SetViewColor(grey);
		statusbar->SetViewColor(grey);
	// adding the view to the window and filling up the view..
		AddChild(view);
		view->AddChild(mainMenuField);
		view->AddChild(okBtn);
		view->AddChild(cancelBtn);
		view->AddChild(browseBtn);
		view->AddChild(editbox);
		view->AddChild(statusbar);
		view->AddChild(historyField);

	// MainMenu tweaks
		mainPopMenu->SetRadioMode(false);
		mainMenuField->SetDivider(40);
		mainPopMenu->AddItem(new BMenuItem("about..",new BMessage(MI_ABOUT)));
		mainPopMenu->AddItem(new BMenuItem("always on top",new BMessage(MI_ONTOP)));

		mainPopMenu->ItemAt(1)->SetMarked(true);

	// tweaking everything once OnView..
		// note the SetModificationMessage must be called AFTER the control is added - otherwise it crushes.
		editbox->SetModificationMessage(new BMessage(TEXT_EDIT));
		editbox->SetDivider(30);
		editbox->MakeFocus(true);
		// editbox->statusbar = statusbar;
	
	// filling up the HistoryPopupMenu, and setting the MenuField's divider.
		fillHistoryMenu(historyPopMenu);
		historyField->SetDivider(0);
	
		// adding my BMessageFilter to editbox's internal BTextView to fetch
		// DELETE, BACKSPACE and ENTER messages (and respond properly)
			filter = new myFilter();
			editbox->TextView()->AddFilter(filter);
}


snapWin :: ~snapWin()
{
	// no. this crashes;
	//	delete filter;
	delete filePanel;
}


bool snapWin :: QuitRequested()
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return BWindow::QuitRequested();
}


// returns a Horizontally centered version of rect
void snapWin :: WHCenter(BRect& rect)
{
	float Cx = Bounds().right / 2;
	float width = rect.right - rect.left;
	
	rect.left = Cx - (rect.right - rect.left)/2;
	rect.right = rect.left + width;
}


// returns a Vertically centered version of rect
void snapWin :: WVCenter(BRect& rect)
{
	float Cy = Bounds().bottom / 2;
	float height = rect.bottom - rect.top;
	
	rect.top = Cy - (rect.bottom - rect.top)/2;
	rect.bottom = rect.top + height;
}


// glues a control to the left of the form (with a bit of space..)
void snapWin :: GlueLeft(BRect& rect, int space)
{
	float width = rect.right - rect.left;
	rect.left = Bounds().left + space;
	rect.right = rect.left + width;
}


// glues a control to the right of the form (with a bit of space..)
void snapWin :: GlueRight(BRect &rect,int space)
{
	float width = rect.right - rect.left;
	rect.right = Bounds().right - space;
	rect.left = rect.right - width;
}


	
	
	// event routing.. (not sure this is the exact term, anyhow..)
	void snapWin :: MessageReceived(BMessage* message)
	{
		switch (message->what)
		{
			case MI_ABOUT:
				aboutBox();
			break;
			
			case MI_ONTOP:
				ontopToggle();
			break;
			
			case TEXT_EDIT:
			{
				ignore_enter = false;
				editbox->TextChanged();
			}
			break;
	
			case ENTER_KEY:
				// note that the flag is determined in the constructor and then in okBtnClick();
				// (once a single launch is successfull.) -
					 // might not be enough with WindowShade?
				if (!ignore_enter) okBtnClick();
			break;
			
			case B_REFS_RECEIVED:
				filePanelClosed(*message);
			break;
			
			case B_CANCEL:
				// ? 
			break;
	
			case BACKSPACE_KEY:
				editbox->DisableCompletion();
			break;
	
			case DELETE_KEY:
				editbox->DisableCompletion();
			break;
	
			case OTHER_KEY:
				editbox->EnableCompletion();
			break;
	
			case HISTORY_ITEM_CLICK:
				historyItemClicked();	
			break;
			
			case OK:
				okBtnClick();
			break;
	
			case BROWSE:
				browseBtnClick();
			break;
	
			case CANCEL:
				QuitRequested();
			break;
	
			default:
				BWindow::MessageReceived(message);
		}
	}


///`````````````````````````````````````	:
// Event functoins												:
//,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,	:


// shows the abouxbox while temporarly disabling the window's OnTop (if so).
void snapWin :: aboutBox()
{
	BAlert* aboutAlert = new BAlert("about","\
snap v1, by m7m (Guy Haviv)\n\nfor Questions, Comments, Leftwing Politic Issues, etc: \n\
mul_m7m@bezeqint.net \n\
m7m_GuyHaviv on BeShare",
		"woop");
		
	aboutAlert->SetFeel(B_MODAL_ALL_WINDOW_FEEL);

	if (IsOnTop) SetFeel(B_NORMAL_WINDOW_FEEL);
	aboutAlert->Go();
	if (IsOnTop) SetFeel(B_FLOATING_ALL_WINDOW_FEEL);	
}


void snapWin :: ontopToggle()
{
	if (IsOnTop)
	{
		SetFeel(B_NORMAL_WINDOW_FEEL);
		IsOnTop=false;
		mainPopMenu->ItemAt(1)->SetMarked(false);
	}
	else
	{
		SetFeel(B_FLOATING_ALL_WINDOW_FEEL);
		IsOnTop=true;		
		mainPopMenu->ItemAt(1)->SetMarked(true);
	}
}


// the Ok Button event. attemps to execute the text in the editbox
void snapWin :: okBtnClick()
{
	ignore_enter = true;
	if (strlen(editbox->Text())>0) 
	{
		entry_ref ref;
		BEntry entry;
		status_t result = B_OK;
		result = get_ref_for_path(editbox->Text(),&ref);
		if (result==B_OK)
		{
			/**/ //WindowShadeColor();
				entry.SetTo(&ref,true);
				if (entry.IsDirectory())
				{
					char *b = (char*)editbox->Text();
					result = be_roster->Launch("application/x-vnd.Be-TRAK",1,&b);
				}
				else
					result = be_roster->Launch(&ref);
					
				if (result==B_OK || result==B_ALREADY_RUNNING)
				{
					createHistoryLink(editbox->Text(),historyPopMenu);
					QuitRequested();
				}
			/**/ // WindowShadeRevert();
		}

		// can either be faliure to get_ref or failure to execute.
		// please note that ignore_enter is set to TRUE on TEXT_EDIT.
		if (!(result==B_OK || result==B_ALREADY_RUNNING))
			showErrorBox();
	}
}


void snapWin :: showErrorBox()
{
	errorBox = new BAlert("error","something's wrong...","oh no!",
		NULL,NULL,B_WIDTH_AS_USUAL,B_STOP_ALERT);
	
	errorBox->SetFeel(B_MODAL_ALL_WINDOW_FEEL);
	// allowing the Alert to be displayed on top, then returning
	// to always-on-top right after...
	if (IsOnTop) SetFeel(B_NORMAL_WINDOW_FEEL);
	errorBox->Go();
	if (IsOnTop) SetFeel(B_FLOATING_ALL_WINDOW_FEEL);
}



// the BrowseButton event. executes a BFilePanel to select a file,
// then places its fullpath on its editbox.
void snapWin :: browseBtnClick()
{
	// note - make the BFilePanel invoke from the currect_path in the editbox.
																				/*entry_ref to panel directory!! */
	filePanel->SetPanelDirectory(editbox->Path().String());
	filePanel->Show();
}


// scans for the marked menu item and copies its text to the editbox
// then selects all text.
void snapWin :: historyItemClicked()
{
	int item_count = historyPopMenu->CountItems();
	int i=0;
	while (i<item_count)
	{
		if (historyPopMenu->ItemAt(i)->IsMarked())
			break;
		i++;
	}
	editbox->SetText(historyPopMenu->ItemAt(i)->Label());
	editbox->TextView()->SelectAll();
}


// copies the fullpath of the selected item in filePanel to the editBox.
void snapWin :: filePanelClosed(BMessage msg)
{
	entry_ref ref;
	if (msg.FindRef("refs",&ref) == B_OK)
	{
		BEntry entry(&ref,false);
		BPath bpath;
		
		if (entry.GetPath(&bpath) == B_OK)
			editbox->UpdateText(bpath.Path());
	}
}


void snapWin::WindowShadeColor()
{
	BString command="";
	command << "WindowShade" <<  " -activetab 567dd4" << " -activeborder 567dd4" ;
	
	system(command.String());
}

void snapWin::WindowShadeRevert()
{
	BString command="";
	command << "WindowShade" << " -activetab ffcb00" << " -activeborder ffcb00";
	
	system(command.String());
}






///`````````````````````````````````````	:
// Service functoins												:
//,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,	:


void createHistoryLink(const char* fullpath, BMenu* menu)
{
	// perhaps cache this later inside the header and just create symlinks .. sounds better.
	// add a check for the existance of the path (all in the ReadyToRun or something..)
	// and create a path if neccesarry.
	// learn some english spelling. <<
	if (!existsInMenu(fullpath, menu))
	{
		BDirectory history_dir(HISTORY_DIR);
		BString link_relative_name="";
		int item_count = menu->CountItems()+1;
		if (item_count<10)
			link_relative_name << "0" << item_count << " - " << fullpath;
		else
			link_relative_name << menu->CountItems()+1 << " - " << fullpath;
			
		link_relative_name.ReplaceAll('/','\\');
		status_t link_result = history_dir.CreateSymLink(link_relative_name.String(),fullpath,NULL);
		if (link_result!=B_OK && link_result!=B_FILE_EXISTS)
			(new BAlert("hono!","problem with creating history symlink","grr. back to the drawing board"))->Go();
			
		menu->AddItem(new BMenuItem(fullpath,new BMessage(HISTORY_ITEM_CLICK)));
	}
}


// scans through the settings_directory and reads all links into history_menu_items.
void fillHistoryMenu(BMenu* menu)
{
	// NOTE: consider reversing the list's order (the largest numbered item is actually the one Invoked last, so i want it First on the list);
	BDirectory history_dir(HISTORY_DIR);
	BEntry entry;
	// BList item_list(0);
	while(history_dir.GetNextEntry(&entry,false) == B_OK)
	{
		char name[B_FILE_NAME_LENGTH];
		entry.GetName(name);
		BString nameString="";
		nameString << name;
		int32 index = nameString.FindFirst(" - ");
		nameString.Remove(0,index + 3);
		nameString.ReplaceAll('\\','/');
	
		menu->AddItem(new BMenuItem(nameString.String(), new BMessage(HISTORY_ITEM_CLICK)))	;
		// item_list.AddItem(&nameString);
	}

	// dump BList and use STL. damnit.

	// IDEA: use the TAutoCompleteTextcontrol:: get_filelist and reverse the internals of the BString :) 

	/* obviously, this doenst work.
	for (int i=item_list.CountItems()-1;i>0;i--)
		menu->AddItem(new BMenuItem(
			((BString*)item_list.ItemAt(i))->String(),
			new BMessage(HISTORY_ITEM_CLICK) ));
	*/
}


// checks if a menuitem with that string exists.
bool existsInMenu(BString string, BMenu* menu)
{
	int i=0;
	int item_count = menu->CountItems();
	while (i<item_count)
 	{
		if (menu->ItemAt(i)->Label() == string)
			return true;
		i++;
	}
	return false;
}